"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withMethodsTemplate = void 0;
exports.withMethodsTemplate = function (queries, mutations) {
    return "\n\n  interface Abortable {\n    $abort(): void;\n  }\n  interface WithArgs<T, A> {\n    $args(args: A): ExecutableQuery<T>;\n  }\n  interface Fragmentable<R> {\n    $fragment(fragment: string | DocumentNode): R;\n  }\n  interface Loadable {\n    loading: boolean;\n  }\n  interface Executable<T> {\n    $fetch(): Promise<T>;\n  }\n  \n  export interface QueryWithArgs<T, A> extends WithArgs<T, A>, Abortable, Loadable {}\n  export interface QueryWithOptionalArgs<T, A> extends QueryWithArgs<T, A>, Executable<T> {}\n  \n  export interface ExecutableQuery<T> extends Abortable, Loadable, Executable<T> {}\n  \n  export interface FragmentableQuery<T, A> extends Fragmentable<ExecutableQuery<T, A>> {}\n  export interface FragmentableQueryWithArgs<T, A> extends Fragmentable<QueryWithArgs<T, A>> {}\n  export interface FragmentableQueryWithOptionalArgs<T, A>\n    extends Fragmentable<QueryWithOptionalArgs<T, A>> {}\n  \n  export interface UnFragmentableQuery<T, A>\n    extends Fragmentable<ExecutableQuery<T, A>>,\n      WithArgs<T,A>,\n      Executable<T, A>,\n      Abortable,\n      Loadable {}\n  export interface UnFragmentableQueryWithArgs<T, A>\n    extends Fragmentable<QueryWithArgs<T, A>>,\n      WithArgs<T,A>,\n      Executable<T, A>,\n      Abortable,\n      Loadable {}\n  export interface UnFragmentableQueryWithOptionalArgs<T, A>\n    extends Fragmentable<QueryWithOptionalArgs<T, A>>,\n      WithArgs<T,A>,\n      Executable<T, A>,\n      Abortable,\n      Loadable {}\n  \n  \n  export const apiProvider = (apolloClient: ApolloClient<any>) => {\n    const abortableQuery = <T, A = null>(\n      query: DocumentNode,\n      args: boolean,\n      optionalArgs: boolean\n    ): A extends null ? AbortableQuery<T> : AbortableQueryWithArgs<T, A> => {\n      let observableQuery: ZenObservable.Subscription;\n      const parsedQuery = query.definitions[0] as OperationDefinitionNode;\n      const queryName = parsedQuery.name.value;\n      let variables: { [x: string]: any } = {};\n      let loading = false;\n\n      function $abort() {\n        if (observableQuery && !observableQuery.closed) {\n          observableQuery.unsubscribe();\n        }\n      }\n      async function $fetch() {\n        loading = true;\n        return new Promise<T>((resolve, reject) => {\n          observableQuery = execute(apolloClient.link, {\n            query,\n            variables,\n          }).subscribe({\n            next: ({ data, errors }) => {\n              if (errors) {\n                reject(errors);\n              } else {\n                resolve(data[queryName]);\n              }\n            },\n            error: (error) => reject(error),\n            complete: () => {\n              loading = false;\n            },\n          });\n        });\n      }\n      function $args(args) {\n        variables = args;\n        return {\n          $abort,\n          $fetch,\n          loading\n        };\n      }\n      if (args && !optionalArgs) {\n        return {\n          $abort,\n          $args,\n          loading\n        } as any;\n      } else if (optionalArgs) {\n        return {\n          $abort,\n          $args,\n          $fetch,\n          loading\n        } as any;\n      } else {\n        return {\n          $abort,\n          $fetch,\n          loading\n        } as any;\n      }\n    };\n\n    return {\n      " + queries.join('\n') + "\n      " + mutations.join('\n') + "\n    }\n  };\n\n  ";
};
